@@ -212,13 +212,13 @@ public final class DecimalVector extends BaseFixedWidthVector
 
     long outAddress = valueBuffer.memoryAddress() + (long) index * TYPE_WIDTH;
     if (length == 0) {
-      MemoryUtil.UNSAFE.setMemory(outAddress, DecimalVector.TYPE_WIDTH, (byte) 0);
+      MemoryUtil.setMemory(outAddress, DecimalVector.TYPE_WIDTH, (byte) 0);
       return;
     }
     if (LITTLE_ENDIAN) {
       // swap bytes to convert BE to LE
       for (int byteIdx = 0; byteIdx < length; ++byteIdx) {
-        MemoryUtil.UNSAFE.putByte(outAddress + byteIdx, value[length - 1 - byteIdx]);
+        MemoryUtil.putByte(outAddress + byteIdx, value[length - 1 - byteIdx]);
       }
 
       if (length == TYPE_WIDTH) {
@@ -228,21 +228,16 @@ public final class DecimalVector extends BaseFixedWidthVector
       if (length < TYPE_WIDTH) {
         // sign extend
         final byte pad = (byte) (value[0] < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
         return;
       }
     } else {
       if (length <= TYPE_WIDTH) {
         // copy data from value to outAddress
-        MemoryUtil.UNSAFE.copyMemory(
-            value,
-            MemoryUtil.BYTE_ARRAY_BASE_OFFSET,
-            null,
-            outAddress + DecimalVector.TYPE_WIDTH - length,
-            length);
+        MemoryUtil.copyToMemory(value, 0, outAddress + DecimalVector.TYPE_WIDTH - length, length);
         // sign extend
         final byte pad = (byte) (value[0] < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
         return;
       }
     }
@@ -281,21 +276,20 @@ public final class DecimalVector extends BaseFixedWidthVector
     long inAddress = buffer.memoryAddress() + start;
     long outAddress = valueBuffer.memoryAddress() + (long) index * TYPE_WIDTH;
     if (LITTLE_ENDIAN) {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress, length);
+      MemoryUtil.copyMemory(inAddress, outAddress, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress + length - 1);
+        byte msb = MemoryUtil.getByte(inAddress + length - 1);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(
-          inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
+      MemoryUtil.copyMemory(inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
       }
     }
   }
@@ -322,23 +316,22 @@ public final class DecimalVector extends BaseFixedWidthVector
     if (LITTLE_ENDIAN) {
       // swap bytes to convert BE to LE
       for (int byteIdx = 0; byteIdx < length; ++byteIdx) {
-        byte val = MemoryUtil.UNSAFE.getByte((inAddress + length - 1) - byteIdx);
-        MemoryUtil.UNSAFE.putByte(outAddress + byteIdx, val);
+        byte val = MemoryUtil.getByte((inAddress + length - 1) - byteIdx);
+        MemoryUtil.putByte(outAddress + byteIdx, val);
       }
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(
-          inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
+      MemoryUtil.copyMemory(inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
       }
     }
   }
